function [XP,y,rp,fi] = p_y_given_x(num_pat,num_indep_feat,num_dep_feat,num_repeat_feat,num_useful_feat,flip_y)
%[XP,y,rp,fi] = p_y_given_x(num_pat,num_indep_feat,num_dep_feat,num_repeat_feat,num_useful_feat,flip_y)
% Draws a pattern recognition problem at random.
% X contains a number of features drawn at random with specified dependencies
% N(0,1) is used to draw the independent features.
% Random dependencies are drawn, also using N(0,1).
% Random noise is added to the features accoding to N(0,.1).
% Features are then shifted and rescaled randomly to span 3 orders of magnitude.
% The y values are obtained using a linear classification from a random weigh vector
% the components of which are drawn according to N(0,1), applied to the useful features
% the have been centered and rescaled to a stdev of 1.
% if flip_y is provided, a random fraction flip_y of labels are flipped.
% Inputs:
%  num_pat -- Number of patterns
%  num_indep_feat -- Number of independent features
%  num_dep_feat -- Number of features linearly dependent upon the independent features
%  num_repeat_feat -- Number of features repeating the previous ones (drawn at random)
%  num_useful_feat -- Number of useful features
%  flip_y -- Fraction of y labels to be flipped at random (number between 0 and 1)
% Returns:
%	XP -- Matrix (num_pat, num_feat) of randomly permuted features
%  y -- Vector of +-1 target values
%  rp -- permutation matrix used (originally the useful features come first)
%  fi -- indices of flipped labels

if (num_useful_feat>num_indep_feat) error('Too many useful features.'); help p_y_given_x; end

num_feat=num_indep_feat+num_dep_feat+num_repeat_feat;
X=zeros(num_pat,num_feat);

% Draw n features independently at random
for i=1:num_indep_feat
   X(:,i)=random('norm', 0, 1, num_pat, 1);
end

% Each additional feature defined as a linear combination of the previous ones
% Provided by a matrix of coefficients of dimention (num_indep_feat, num_dep_feat)
A = 2*rand(num_indep_feat, num_dep_feat)-1;
X(:,num_indep_feat+1:num_indep_feat+num_dep_feat)=X(:,1:num_indep_feat)*A;

% Repeat num_repeat_feat features, chosen at random among indep and dep feat
nf=num_indep_feat+num_dep_feat;
rf=round(1+rand(num_repeat_feat,1)*(nf-1));
X(:,nf+1:num_feat)=X(:,rf);

% Add a little bit of random noise
N=random('norm', 0, .1, num_pat, num_feat);
X=X+N;

% Randomly shift and scale
shift=rand(num_feat,1);
scale=1+1000*rand(num_feat,1);
X=X+repmat(shift',num_pat,1);
X=X.*repmat(scale',num_pat,1);

% Draw y according to a random weight vector using only the useful feat, centered  about their mean
% center and scale features
XX=normalize_data(X(:,1:num_useful_feat));
w=rand(num_useful_feat,1);
y=sign(XX*w);

% Add random y label errors
num_err_pat = round(num_pat*flip_y);
rp=randperm(num_pat);
fi=rp(1:num_err_pat);
y(fi)=y(fi)*-1;

% sort to display
[ys, is]=sort(y);
featdisplay([normalize_data(X(is,:)),ys]);

% randomly permute the features
rp=randperm(num_feat);
XP=X(:,rp);
